import time

from .Database import DatabaseEngine
from .Executors import InfoQueryExecutor


class ResponseTimeExecutor(InfoQueryExecutor):
    def get_statistic(self, query, cursor, database):
        start = time.perf_counter()
        cursor.execute("select * from loadbalancer_destination limit 1")
        result = cursor.fetchall()
        end = time.perf_counter()
        return end - start


class NumberOfConnectionsExecutor(InfoQueryExecutor):
    def get_statistic(self, query, cursor, database):
        cursor.execute(self.Generator.generate(database))
        return self.Parser.parse(cursor.fetchall(), database)

    class Parser:
        @staticmethod
        def parse(result, database):
            if database.engine == DatabaseEngine.POSTGRESQL.value:
                return result[0][0]

    class Generator:
        @staticmethod
        def generate(database):
            if database.engine == DatabaseEngine.POSTGRESQL.value:
                return "select count(*) from pg_stat_activity where pid <> pg_backend_pid() and usename = current_user;"



import os
import platform
import sys
from pathlib import Path
PATH_TO_SETTINGS=''
for dirpath, dirnames, files in os.walk('./'):
    if dirpath[len('./'):].count(os.sep) < 1:
        if 'settings.py' in files:
            PATH_TO_SETTINGS = dirpath[2:] + '.settings'
if platform.system()=='Windows':
    sys.path.append(str(Path(__file__).parent.parent.parent.parent.parent) + '\\')
else:
    sys.path.append(str(Path(__file__).parent.parent.parent.parent.parent.parent) + '/')
os.environ.setdefault("DJANGO_SETTINGS_MODULE", PATH_TO_SETTINGS)
import django

django.setup()

from django.conf import settings

LOAD_BALANCER = settings.LOAD_BALANCER
DATABASES = settings.DATABASES


import django
import django.db

from .Database import DatabaseStatus


class Waiter:
    def __init__(self, executor=None):
        self._executor = executor

    def run_query(self, query, database):
        pass


class WaitWaiter(Waiter):
    def __init__(self, executor=None):
        super(WaitWaiter, self).__init__(executor)

    def run_query(self, query, database):
        if database.check_status() == DatabaseStatus.RUNNING.value:
            try:
                self._executor.run_query(query, database)
            except django.db.utils.OperationalError:
                database.is_up.clear()
                database.change_status(DatabaseStatus.DOWN.value)
                self.wait_for_connection(query, database)
        else:
            self.wait_for_connection(query, database)

    def wait_for_connection(self, query, database):
        database.is_up.wait()
        self.run_query(query,database)


class DontWaitWaiter(Waiter):
    def run_query(self, query, database):
        if database.check_status() == DatabaseStatus.RUNNING.value:
            try:
                self._executor.run_query(query, database)
            except django.db.utils.OperationalError:
                database.is_up.clear()
                database.change_status(DatabaseStatus.DOWN.value)
                raise django.db.utils.OperationalError



import sys

import django.db.utils

from .Query import Wait


class LoadBalancer():
    def __init__(self, result, databases, cud_algorithm, r_algorithm,watch_dog):
        self.current_query_id = 0
        self.result = result
        self.databases = databases
        self.cud_algorithm = cud_algorithm
        self.r_algorithm = r_algorithm
        self.watch_dog=watch_dog

    def run_query(self, query):
        self.generate_query_id()
        query.query_id = self.current_query_id
        if query.wait == Wait.DONT_WAIT.value:
            self.execute_r(query)
        else:
            self.execute_cud(query)
        res = self.result.get()
        while list(res.keys())[0] != self.current_query_id:
            res = self.result.get()
        return list(res.values())[0]

    def execute_cud(self, query):
        self.cud_algorithm.execute_cud(query)

    def execute_r(self, query):
        while True:
            try:
                self.r_algorithm.execute_r(query)
            except django.db.utils.OperationalError:
                pass
            else:
                break

    def generate_query_id(self):
        if self.current_query_id == sys.maxsize:
            self.current_query_id = 0
        else:
            self.current_query_id += 1



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    LONG_DESCRIPTION = fh.read()

VERSION = '0.0.36'
PACKAGE_NAME = 'djangoloadbalancer'
AUTHOR = 'Daniel Klarenbach'
URL = 'https://github.com/DanielKlarenbach/DjangoLoadBalancer'
LICENSE = 'Apache License 2.0'
DESCRIPTION = 'Package enabling to load balance requests to databases in django projects'
LONG_DESC_TYPE = 'text/markdown'
INSTALL_REQUIRES = [

]

setuptools.setup(
    name=PACKAGE_NAME,
    version=VERSION,
    author=AUTHOR,
    url=URL,
    license=LICENSE,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type=LONG_DESC_TYPE,
    install_requires=INSTALL_REQUIRES,
    packages=setuptools.find_packages(),
    scripts=['bin/djangoloadbalancer.py'],
)
